@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_sel_cmd.erl,v 1.63 2005/08/15 22:36:34 dgud Exp $
+%%     $Id: wings_sel_cmd.erl,v 1.66 2006/07/10 10:56:15 giniu Exp $
 %%
 
 -module(wings_sel_cmd).
@@ -71,13 +71,19 @@ menu(St) ->
 	     [{?__(37,"2 Edges"),2},
 	      {?__(38,"3 Edges"),3},
 	      {?__(39,"4 Edges"),4},
-	      {?__(40,"5 Edges"),5}]}},
+	      {?__(40,"5 Edges"),5},
+	      {?__(401,"6 or More"),6}]}},
 	   {?__(41,"Faces With"),
 	    {faces_with,
 	     [{?__(42,"2 Edges"),2},
 	      {?__(43,"3 Edges"),3},
 	      {?__(44,"4 Edges"),4},
 	      {?__(45,"5 or More"),5}]}},
+	   {?__(nq0,"Non Quadrangle Faces"),
+	    {non_quad,
+	     [{?__(nq1,"All Non Quadrangle Faces"),all},
+	      {?__(nq2,"Odd Non Quadrangle Faces"),odd},
+	      {?__(nq3,"Even Non Quadrangle Faces"),even}]}},
 	   {?__(46,"Random"),
 	    {random,[{"10%",10},
 		     {"20%",20},
@@ -90,6 +96,8 @@ menu(St) ->
 		     {"90%",90}]}},
 	   {?__(56,"Short Edges"),
 	    short_edges,?__(57,"Select (too) short edges"),[option]},
+           {?__(87,"Sharp Edges"),
+            sharp_edges,?__(88,"Select sharp edges"),[option]},
 	   {?__(58,"Material Edges"),material_edges,
 	    ?__(59,"Select all edges between different materials")},
 	   {?__(60,"UV-Mapped Faces"),uv_mapped_faces,
@@ -294,6 +302,15 @@ by_command(isolated_vertices, St) ->
     {save_state,select_isolated(St)};
 by_command({nonplanar_faces,Ask}, St) ->
     nonplanar_faces(Ask, St);
+by_command({vertices_with,6}, St) ->
+    Sel = fun(V, We) ->
+		  Cnt = wings_vertex:fold(
+			  fun(_, _, _, Cnt) ->
+				  Cnt+1
+			  end, 0, V, We),
+		  Cnt >= 6
+	  end, 
+    {save_state,wings_sel:make(Sel, vertex, St)};
 by_command({vertices_with,N}, St) ->
     Sel = fun(V, We) ->
 		  Cnt = wings_vertex:fold(
@@ -303,6 +320,23 @@ by_command({vertices_with,N}, St) ->
 		  Cnt =:= N
 	  end, 
     {save_state,wings_sel:make(Sel, vertex, St)};
+by_command({non_quad,all}, St) ->
+    Sel = fun(Face, We) ->
+		  wings_face:vertices(Face, We) =/= 4
+	    end,
+    {save_state,wings_sel:make(Sel, face, St)};
+by_command({non_quad,odd}, St) ->
+    Sel = fun(Face, We) ->
+                  Half = wings_face:vertices(Face, We)/2,
+		  trunc(Half) /= Half
+	    end,
+    {save_state,wings_sel:make(Sel, face, St)};
+by_command({non_quad,even}, St) ->
+    Sel = fun(Face, We) ->
+                  Half = wings_face:vertices(Face, We)/2,
+		  (Half /= 2) and (trunc(Half) == Half)
+	    end,
+    {save_state,wings_sel:make(Sel, face, St)};
 by_command({faces_with,5}, St) ->
     Sel = fun(Face, We) ->
 		  wings_face:vertices(Face, We) >= 5
@@ -319,6 +353,8 @@ by_command({random,Percent}, St) ->
     {save_state,random(Percent, St)};
 by_command({short_edges,Ask}, St) ->
     short_edges(Ask, St);
+by_command({sharp_edges,Ask}, St) ->
+    sharp_edges(Ask, St);
 by_command(uv_mapped_faces, St) ->
     uv_mapped_faces(St);
 by_command(id, St) ->
@@ -473,9 +509,14 @@ combine_sel(Combine, [{Id,S0}|T]) ->
 combine_sel(_Combine, []) -> [].
 
 coerce_ssel({Mode,_}=Key, #st{ssels=Ssels}=St) ->
-    Ssel = gb_trees:get(Key, Ssels),
-    coerce_ssel(Mode, Ssel, St).
-    
+    case gb_trees:is_defined(Key,Ssels) of
+	true -> 
+	    Ssel = gb_trees:get(Key, Ssels),
+	    coerce_ssel(Mode, Ssel, St);
+	false ->
+	    []
+    end.
+
 coerce_ssel(Mode, Ssel, #st{selmode=Mode}) -> Ssel;
 coerce_ssel(Smode, Ssel0, #st{selmode=Mode}=St) ->
     StTemp = St#st{selmode=Smode,sel=wings_sel:valid_sel(Ssel0, Smode, St)},
@@ -881,3 +922,28 @@ any_matching_normal(CosTolerance, Normal, [N|T]) ->
       Dot >= CosTolerance -> true;
       true -> any_matching_normal(CosTolerance, Normal, T)
     end.
+
+%%%
+%%% Select sharp edges
+%%%
+
+sharp_edges(Ask, _St) when is_atom(Ask) ->
+    Qs = [{label,?__(1,"Max Angle")},
+          {text,120.0,[{range,{0.0,180.0}}]}],
+    wings_ask:dialog(Ask, 
+        ?__(2,"Select Sharp Edges"), [{hframe,Qs}],
+        fun(Res) ->
+            {select,{by,{sharp_edges,Res}}} 
+        end);
+sharp_edges([Tolerance], St0) ->
+    CosTolerance = -math:cos(Tolerance * math:pi() / 180.0),
+    St = wings_sel:make(fun(Edge, We) ->
+             sharp_edge(CosTolerance, Edge, We)
+         end, edge, St0),
+    {save_state,St}.
+
+sharp_edge(CosTolerance, Edge, #we{es=Etab}=We) ->
+    #edge{lf=Lf,rf=Rf} = gb_trees:get(Edge, Etab),
+    Lfn = wings_face:normal(Lf, Edge, We),
+    Rfn = wings_face:normal(Rf, Edge, We),
+    e3d_vec:dot(Lfn,Rfn) < CosTolerance.
